use std::{collections::HashMap, str::from_utf8, vec};

use argh::FromArgs;
use rand::seq::SliceRandom;
use rust_embed::Embed;

static ARTICLE: &str = "<article>";
static GENDER_M: &str = "masculins";
static GENDER_F: &str = "feminins";
static RANDOM: &str = "random";

#[derive(FromArgs)]
#[argh(description = "arguments")]
struct Arguments {
    #[argh(option, description = "nombre de noms à générer")]
    qty: Option<u8>,
    #[argh(option, description = "separateur")]
    sep: Option<String>,
    #[argh(option, description = "gender")]
    gender: Option<String>,
    #[argh(option, description = "sequence")]
    sequence: Option<String>,
}

#[derive(Embed)]
#[folder = "dictionaries/"]
struct Dictionaries;

fn main() {
    // Load the dictionaries into memory
    let mut dictionaries = HashMap::new();
    for filename in Dictionaries::iter() {
        if let Some(file) = Dictionaries::get(&filename) {
            let data = from_utf8(file.data.as_ref()).unwrap();
            dictionaries.insert(
                filename.to_string(),
                serde_json::from_str::<Vec<String>>(data).unwrap(),
            );
        }
    }

    let args: Arguments = argh::from_env();
    let qty = args.qty.unwrap_or(10).min(50).max(1);
    let sep = args.sep.unwrap_or(" ".to_string());
    let gender = match args.gender.as_ref().map(String::as_ref) {
        Some("m") => GENDER_M,
        Some("f") => GENDER_F,
        _ => RANDOM,
    };

    // Limit to 10 parts max
    let q_dicts = args.sequence.unwrap_or("prenom patronyme".to_string());
    let mut chosen_dicts = q_dicts.split(" ").collect::<Vec<_>>();
    chosen_dicts.truncate(10);

    let mut names = vec![];

    for _ in 0..qty {
        let mut parts = vec![];

        let gender = if gender == RANDOM {
            vec![GENDER_M, GENDER_F]
                .choose(&mut rand::thread_rng())
                .unwrap()
        } else {
            gender
        };

        for dict in chosen_dicts.iter() {
            let filename = match *dict {
                "adjectif" | "commun" | "ppasse" | "ppresent" | "prenom" => {
                    Some(format!("{dict}s_{gender}.json"))
                }
                "patronyme" => Some(format!("{dict}s.json")),
                "article" => Some(ARTICLE.to_string()),
                "virgule" => Some(",".to_string()),
                _ => None,
            };
            if let Some(filename) = filename {
                if let Some(data) = dictionaries.get(&filename) {
                    let item = data.choose(&mut rand::thread_rng()).unwrap();
                    parts.push(item.to_string())
                } else {
                    parts.push(filename)
                }
            }
        }

        // Replace the <article> with le/la/l'
        let mut to_replace = vec![];
        for (i, item) in parts.iter().enumerate() {
            if item == ARTICLE {
                // Check the next word
                if let Some(next) = parts.get(i + 1) {
                    if next.starts_with(|c| {
                       vec!['a', 'e', 'i', 'o', 'u', 'é', 'è', 'ê', 'h'].contains(&c)
                   } && !next.starts_with("hy")) {
                       to_replace.push((i, "l'"));
                   } else {
                       to_replace.push((i, (if gender == "masculins" { "le" } else { "la" })))
                   }
                }
            }
        }
        for item in to_replace.iter() {
            parts[item.0] = item.1.to_string();
        }

        let joined = parts
            .join(&sep)
            // Remove space after "l'"
            .replace("l' ", "l'")
            // Remove space before comma
            .replace(" ,", ",");

        names.push(joined);
    }
    println!("{}", serde_json::to_string(&names).unwrap());
}
